@@ -24,6 +24,7 @@ sys.dont_write_bytecode = True
 
 import json
 import os
+import random
 import subprocess
 from ci_settings import (
     PG14_EARLIEST,
@@ -32,6 +33,8 @@ from ci_settings import (
     PG15_LATEST,
     PG16_EARLIEST,
     PG16_LATEST,
+    PG17_EARLIEST,
+    PG17_LATEST,
     PG_LATEST,
 )
 
@@ -51,6 +54,16 @@ default_ignored_tests = {
     "memoize",
 }
 
+# Tests that we do not run as part of a Flake tests
+flaky_exclude_tests = {
+    # Not executed as a flake test since it easily exhausts available
+    # background worker slots.
+    "bgw_launcher",
+    # Not executed as a flake test since it takes a very long time and
+    # easily interferes with other tests.
+    "bgw_scheduler_restart",
+}
+
 
 # helper functions to generate matrix entries
 # the release and apache config inherit from the
@@ -77,7 +90,7 @@ def build_debug_config(overrides):
             "name": "Debug",
             "os": "ubuntu-22.04",
             "pg_extra_args": "--enable-debug --enable-cassert --with-llvm LLVM_CONFIG=llvm-config-14",
-            "pg_extensions": "postgres_fdw test_decoding",
+            "pg_extensions": "postgres_fdw test_decoding pageinspect pgstattuple",
             "pginstallcheck": True,
             "tsdb_build_args": "-DWARNINGS_AS_ERRORS=ON -DREQUIRE_ALL_TESTS=ON",
         }
@@ -91,7 +104,6 @@ def build_debug_config(overrides):
 # builds. This will capture some cases where warnings are generated
 # for release builds but not for debug builds.
 def build_release_config(overrides):
-    base_config = build_debug_config({})
     release_config = dict(
         {
             "name": "Release",
@@ -100,26 +112,24 @@ def build_release_config(overrides):
             "coverage": False,
         }
     )
-    base_config.update(release_config)
-    base_config.update(overrides)
-    return base_config
+    release_config.update(overrides)
+    return build_debug_config(release_config)
 
 
 def build_without_telemetry(overrides):
-    config = build_release_config({})
-    config.update(
+    config = dict(
         {
             "name": "ReleaseWithoutTelemetry",
-            "tsdb_build_args": config["tsdb_build_args"] + " -DUSE_TELEMETRY=OFF",
             "coverage": False,
         }
     )
     config.update(overrides)
+    config = build_release_config(config)
+    config["tsdb_build_args"] += " -DUSE_TELEMETRY=OFF"
     return config
 
 
 def build_apache_config(overrides):
-    base_config = build_debug_config({})
     apache_config = dict(
         {
             "name": "ApacheOnly",
@@ -128,9 +138,8 @@ def build_apache_config(overrides):
             "coverage": False,
         }
     )
-    base_config.update(apache_config)
-    base_config.update(overrides)
-    return base_config
+    apache_config.update(overrides)
+    return build_debug_config(apache_config)
 
 
 def macos_config(overrides):
@@ -149,7 +158,7 @@ def macos_config(overrides):
             "ignored_tests": default_ignored_tests.union(macos_ignored_tests),
             "os": "macos-13",
             "pg_extra_args": "--with-libraries=/usr/local/opt/openssl@3/lib --with-includes=/usr/local/opt/openssl@3/include --without-icu",
-            "pg_extensions": "postgres_fdw test_decoding",
+            "pg_extensions": "postgres_fdw test_decoding pageinspect pgstattuple",
             "pginstallcheck": True,
             "tsdb_build_args": "-DASSERTIONS=ON -DREQUIRE_ALL_TESTS=ON -DOPENSSL_ROOT_DIR=/usr/local/opt/openssl@3",
         }
@@ -165,15 +174,17 @@ m["include"].append(build_debug_config({"pg": PG15_LATEST}))
 
 m["include"].append(build_debug_config({"pg": PG16_LATEST}))
 
+m["include"].append(build_debug_config({"pg": PG17_LATEST}))
+
 # test timescaledb with release config on latest postgres release in MacOS
-m["include"].append(build_release_config(macos_config({"pg": PG16_LATEST})))
+m["include"].append(build_release_config(macos_config({"pg": PG17_LATEST})))
 
 # Test latest postgres release without telemetry. Also run clang-tidy on it
 # because it's the fastest one.
 m["include"].append(
     build_without_telemetry(
         {
-            "pg": PG16_LATEST,
+            "pg": PG17_LATEST,
             "cc": "clang-14",
             "cxx": "clang++-14",
             "tsdb_build_args": "-DLINTER=ON -DWARNINGS_AS_ERRORS=ON",
@@ -181,17 +192,6 @@ m["include"].append(
     )
 )
 
-m["include"].append(
-    build_debug_config(
-        {
-            "pg": "17",
-            "snapshot": "snapshot",
-            "tsdb_build_args": "-DEXPERIMENTAL=ON",
-            "skipped_tests": "merge_compress merge_dml merge size_utils ts_merge-17",
-        }
-    )
-)
-
 # if this is not a pull request e.g. a scheduled run or a push
 # to a specific branch like prerelease_test we add additional
 # entries to the matrix
@@ -212,15 +212,22 @@ if not pull_request:
     # add debug test for first supported PG16 version
     m["include"].append(build_debug_config({"pg": PG16_EARLIEST}))
 
+    # add debug test for first supported PG17 version
+    if PG17_EARLIEST != PG17_LATEST:
+        m["include"].append(build_debug_config({"pg": PG17_EARLIEST}))
+
     # add debug tests for timescaledb on latest postgres release in MacOS
     m["include"].append(build_debug_config(macos_config({"pg": PG15_LATEST})))
 
     m["include"].append(build_debug_config(macos_config({"pg": PG16_LATEST})))
 
+    m["include"].append(build_debug_config(macos_config({"pg": PG17_LATEST})))
+
     # add release test for latest pg releases
     m["include"].append(build_release_config({"pg": PG14_LATEST}))
     m["include"].append(build_release_config({"pg": PG15_LATEST}))
     m["include"].append(build_release_config({"pg": PG16_LATEST}))
+    m["include"].append(build_release_config({"pg": PG17_LATEST}))
 
     # add apache only test for latest pg versions
     for PG_LATEST_VER in PG_LATEST:
@@ -252,6 +259,14 @@ if not pull_request:
             }
         )
     )
+    m["include"].append(
+        build_debug_config(
+            {
+                "pg": 17,
+                "snapshot": "snapshot",
+            }
+        )
+    )
 elif len(sys.argv) > 2:
     # Check if we need to check for the flaky tests. Determine which test files
     # have been changed in the PR. The sql files might include other files that
@@ -301,13 +316,27 @@ elif len(sys.argv) > 2:
             sys.exit(1)
 
     if tests:
+        to_run = [t for t in list(tests) if t not in flaky_exclude_tests] * 20
+        random.shuffle(to_run)
+        installcheck_args = f'TESTS="{" ".join(to_run)}"'
         m["include"].append(
             build_debug_config(
                 {
                     "coverage": False,
-                    "installcheck_args": f'TESTS="{" ".join(list(tests) * 20)}"',
+                    "installcheck_args": installcheck_args,
                     "name": "Flaky Check Debug",
-                    "pg": PG14_LATEST,
+                    "pg": PG16_LATEST,
+                    "pginstallcheck": False,
+                }
+            )
+        )
+        m["include"].append(
+            build_debug_config(
+                {
+                    "coverage": False,
+                    "installcheck_args": installcheck_args,
+                    "name": "Flaky Check Debug",
+                    "pg": PG17_LATEST,
                     "pginstallcheck": False,
                 }
             )
