@@ -33,6 +33,7 @@
 #include <utils/timestamp.h>
 
 #include "compat/compat.h"
+#include "bgw/job_stat_history.h"
 #include "bgw/scheduler.h"
 #include "bgw_policy/chunk_stats.h"
 #include "bgw_policy/policy.h"
@@ -1151,6 +1152,7 @@ ts_bgw_job_entrypoint(PG_FUNCTION_ARGS)
 	INSTR_TIME_SET_CURRENT(start);
 
 	StartTransactionCommand();
+
 	/* Grab a session lock on the job row to prevent concurrent deletes. Lock is released
 	 * when the job process exits */
 	job = ts_bgw_job_find_with_lock(params.job_id,
@@ -1159,14 +1161,16 @@ ts_bgw_job_entrypoint(PG_FUNCTION_ARGS)
 									SESSION_LOCK,
 									/* block */ true,
 									&got_lock);
-	CommitTransactionCommand();
-
 	if (job == NULL)
+		/* If the job is not found, we can't proceed */
 		elog(ERROR, "job %d not found when running the background worker", params.job_id);
 
 	/* get parameters from bgworker */
 	job->job_history.id = params.job_history_id;
 	job->job_history.execution_start = params.job_history_execution_start;
+	ts_bgw_job_stat_history_update(JOB_STAT_HISTORY_UPDATE_PID, job, JOB_SUCCESS, NULL);
+
+	CommitTransactionCommand();
 
 	elog(DEBUG2, "job %d (%s) found", params.job_id, NameStr(job->fd.application_name));
 
