@@ -650,7 +650,7 @@ ts_bgw_job_stat_mark_start(BgwJob *job)
 	job->job_history.execution_start = ts_timer_get_current_timestamp();
 	job->job_history.id = INVALID_BGW_JOB_STAT_HISTORY_ID;
 
-	ts_bgw_job_stat_history_mark_start(job);
+	ts_bgw_job_stat_history_update(JOB_STAT_HISTORY_UPDATE_START, job, JOB_SUCCESS, NULL);
 
 	pgstat_report_activity(STATE_IDLE, NULL);
 }
@@ -674,7 +674,7 @@ ts_bgw_job_stat_mark_end(BgwJob *job, JobResult result, Jsonb *edata)
 				 errmsg("unable to find job statistics for job %d", job->fd.id)));
 	}
 
-	ts_bgw_job_stat_history_mark_end(job, result, edata);
+	ts_bgw_job_stat_history_update(JOB_STAT_HISTORY_UPDATE_END, job, result, edata);
 
 	pgstat_report_activity(STATE_IDLE, NULL);
 }
@@ -693,7 +693,7 @@ ts_bgw_job_stat_mark_crash_reported(BgwJob *job, JobResult result)
 				 errmsg("unable to find job statistics for job %d", job->fd.id)));
 	}
 
-	ts_bgw_job_stat_history_mark_end(job, result, NULL);
+	ts_bgw_job_stat_history_update(JOB_STAT_HISTORY_UPDATE_END, job, result, NULL);
 
 	pgstat_report_activity(STATE_IDLE, NULL);
 }
