@@ -11,12 +11,14 @@
 #include <commands/defrem.h>
 #include <commands/explain.h>
 #include <commands/trigger.h>
+#include <commands/vacuum.h>
 #include <executor/executor.h>
 #include <executor/tuptable.h>
 #include <nodes/execnodes.h>
 #include <nodes/nodes.h>
 #include <optimizer/restrictinfo.h>
 #include <pgstat.h>
+#include <storage/lmgr.h>
 #include <utils/jsonb.h>
 #include <utils/lsyscache.h>
 #include <utils/rel.h>
@@ -25,10 +27,19 @@
 
 #define PG_MAJOR_MIN 14
 
-#define is_supported_pg_version_14(version) ((version >= 140000) && (version < 150000))
-#define is_supported_pg_version_15(version) ((version >= 150000) && (version < 160000))
-#define is_supported_pg_version_16(version) ((version >= 160000) && (version < 170000))
-#define is_supported_pg_version_17(version) ((version >= 170000) && (version < 180000))
+/*
+ * Prevent building against upstream versions that had ABI breaking change (14.14, 15.9, 16.5, 17.1)
+ * that was reverted in the following release.
+ */
+
+#define is_supported_pg_version_14(version)                                                        \
+	((version >= 140000) && (version < 150000) && (version != 140014))
+#define is_supported_pg_version_15(version)                                                        \
+	((version >= 150000) && (version < 160000) && (version != 150009))
+#define is_supported_pg_version_16(version)                                                        \
+	((version >= 160000) && (version < 170000) && (version != 160005))
+#define is_supported_pg_version_17(version)                                                        \
+	((version >= 170000) && (version < 180000) && (version != 170001))
 
 /*
  * PG16 support is a WIP and not complete yet.
@@ -56,6 +67,47 @@
 #error "Unsupported PostgreSQL version"
 #endif
 
+#if ((PG_VERSION_NUM >= 140014 && PG_VERSION_NUM < 150000) ||                                      \
+	 (PG_VERSION_NUM >= 150009 && PG_VERSION_NUM < 160000) ||                                      \
+	 (PG_VERSION_NUM >= 160005 && PG_VERSION_NUM < 170000) || (PG_VERSION_NUM >= 170001))
+/*
+ * The above versions introduced a fix for potentially losing updates to
+ * pg_class and pg_database due to inplace updates done to those catalog
+ * tables by PostgreSQL. The fix requires taking a lock on the tuple via
+ * SearchSysCacheLocked1(). For older PG versions, we just map the new
+ * function to the unlocked version and the unlocking of the tuple is a noop.
+ *
+ * https://github.com/postgres/postgres/commit/3b7a689e1a805c4dac2f35ff14fd5c9fdbddf150
+ *
+ * Here's an excerpt from README.tuplock that explains the need for additional
+ * tuple locks:
+ *
+ * If IsInplaceUpdateRelation() returns true for a table, the table is a
+ * system catalog that receives systable_inplace_update_begin() calls.
+ * Preparing a heap_update() of these tables follows additional locking rules,
+ * to ensure we don't lose the effects of an inplace update. In particular,
+ * consider a moment when a backend has fetched the old tuple to modify, not
+ * yet having called heap_update(). Another backend's inplace update starting
+ * then can't conclude until the heap_update() places its new tuple in a
+ * buffer. We enforce that using locktags as follows. While DDL code is the
+ * main audience, the executor follows these rules to make e.g. "MERGE INTO
+ * pg_class" safer. Locking rules are per-catalog:
+ *
+ * pg_class heap_update() callers: before copying the tuple to modify, take a
+ * lock on the tuple, a ShareUpdateExclusiveLock on the relation, or a
+ * ShareRowExclusiveLock or stricter on the relation.
+ */
+#define SYSCACHE_TUPLE_LOCK_NEEDED 1
+#define AssertSufficientPgClassUpdateLockHeld(relid)                                               \
+	Assert(CheckRelationOidLockedByMe(relid, ShareUpdateExclusiveLock, false) ||                   \
+		   CheckRelationOidLockedByMe(relid, ShareRowExclusiveLock, true));
+#define UnlockSysCacheTuple(rel, tid) UnlockTuple(rel, tid, InplaceUpdateTupleLock);
+#else
+#define SearchSysCacheLockedCopy1(rel, datum) SearchSysCacheCopy1(rel, datum)
+#define UnlockSysCacheTuple(rel, tid)
+#define AssertSufficientPgClassUpdateLockHeld(relid)
+#endif
+
 /*
  * The following are compatibility functions for different versions of
  * PostgreSQL. Each compatibility function (or group) has its own logic for
@@ -366,74 +418,54 @@ pg_strtoint64(const char *str)
 	check_index_is_clusterable(rel, indexOid, true, lock)
 #endif
 
+#if PG16_LT
 /*
  * PG15 consolidate VACUUM xid cutoff logic.
  *
  * https://github.com/postgres/postgres/commit/efa4a946
+ *
+ * PG16 introduced VacuumCutoffs so define here for previous PG versions.
  */
-#if PG15_LT
-#define vacuum_set_xid_limits_compat(rel,                                                          \
-									 freeze_min_age,                                               \
-									 freeze_table_age,                                             \
-									 multixact_freeze_min_age,                                     \
-									 multixact_freeze_table_age,                                   \
-									 oldestXmin,                                                   \
-									 freezeLimit,                                                  \
-									 multiXactCutoff)                                              \
-	vacuum_set_xid_limits(rel,                                                                     \
-						  freeze_min_age,                                                          \
-						  freeze_table_age,                                                        \
-						  multixact_freeze_min_age,                                                \
-						  multixact_freeze_table_age,                                              \
-						  oldestXmin,                                                              \
-						  freezeLimit,                                                             \
-						  NULL,                                                                    \
-						  multiXactCutoff,                                                         \
-						  NULL)
-#elif PG16_LT
-#define vacuum_set_xid_limits_compat(rel,                                                          \
-									 freeze_min_age,                                               \
-									 freeze_table_age,                                             \
-									 multixact_freeze_min_age,                                     \
-									 multixact_freeze_table_age,                                   \
-									 oldestXmin,                                                   \
-									 freezeLimit,                                                  \
-									 multiXactCutoff)                                              \
-	do                                                                                             \
-	{                                                                                              \
-		MultiXactId oldestMxact;                                                                   \
-		vacuum_set_xid_limits(rel,                                                                 \
-							  freeze_min_age,                                                      \
-							  freeze_table_age,                                                    \
-							  multixact_freeze_min_age,                                            \
-							  multixact_freeze_table_age,                                          \
-							  oldestXmin,                                                          \
-							  &oldestMxact,                                                        \
-							  freezeLimit,                                                         \
-							  multiXactCutoff);                                                    \
-	} while (0)
-#else
-#define vacuum_set_xid_limits_compat(rel,                                                          \
-									 freezeMinAge,                                                 \
-									 freezeTableAge,                                               \
-									 multixactFreezeMinAge,                                        \
-									 multixactFreezeTableAge,                                      \
-									 oldestXmin,                                                   \
-									 freezeLimit,                                                  \
-									 multiXactCutoff)                                              \
-	do                                                                                             \
-	{                                                                                              \
-		struct VacuumCutoffs cutoffs;                                                              \
-		/* vacuum_get_cutoffs uses only the *_age members of the VacuumParams object */            \
-		VacuumParams params = { .freeze_min_age = freezeMinAge,                                    \
-								.freeze_table_age = freezeTableAge,                                \
-								.multixact_freeze_min_age = multixactFreezeMinAge,                 \
-								.multixact_freeze_table_age = multixactFreezeTableAge };           \
-		vacuum_get_cutoffs(rel, &params, &cutoffs);                                                \
-		*(oldestXmin) = cutoffs.OldestXmin;                                                        \
-		*(freezeLimit) = cutoffs.FreezeLimit;                                                      \
-		*(multiXactCutoff) = cutoffs.MultiXactCutoff;                                              \
-	} while (0)
+struct VacuumCutoffs
+{
+	TransactionId relfrozenxid;
+	MultiXactId relminmxid;
+	TransactionId OldestXmin;
+	MultiXactId OldestMxact;
+	TransactionId FreezeLimit;
+	MultiXactId MultiXactCutoff;
+};
+
+static inline bool
+vacuum_get_cutoffs(Relation rel, const VacuumParams *params, struct VacuumCutoffs *cutoffs)
+{
+#if PG15
+	return vacuum_set_xid_limits(rel,
+								 0,
+								 0,
+								 0,
+								 0,
+								 &cutoffs->OldestXmin,
+								 &cutoffs->OldestMxact,
+								 &cutoffs->FreezeLimit,
+								 &cutoffs->MultiXactCutoff);
+#elif PG14
+	vacuum_set_xid_limits(rel,
+						  0,
+						  0,
+						  0,
+						  0,
+						  &cutoffs->OldestXmin,
+						  &cutoffs->FreezeLimit,
+						  NULL,
+						  &cutoffs->MultiXactCutoff,
+						  NULL);
+
+	/* Should aggressive vacuum be done? PG14 doesn't support the return value
+	 * so return false. */
+	return false;
+#endif
+}
 #endif
 
 /*
@@ -604,6 +636,12 @@ RelationGetSmgr(Relation rel)
 	GenerationContextCreate(parent, name, blockSize)
 #endif
 
+#if PG16_GE
+#define pgstat_get_local_beentry_by_index_compat(idx) pgstat_get_local_beentry_by_index(idx)
+#else
+#define pgstat_get_local_beentry_by_index_compat(idx) pgstat_fetch_stat_local_beentry(idx)
+#endif
+
 /*
  * PG16 adds a new parameter to DefineIndex, total_parts, that takes
  * in the total number of direct and indirect partitions of the relation.
@@ -920,3 +958,16 @@ RestrictSearchPath(void)
 				 is_internal,                                                                      \
 				 constraintId)
 #endif
+
+#if PG17_LT
+/*
+ * Overflow-aware comparison functions to be used in qsort. Introduced in PG
+ * 17 and included here for older PG versions.
+ */
+static inline int
+pg_cmp_u32(uint32 a, uint32 b)
+{
+	return (a > b) - (a < b);
+}
+
+#endif
