@@ -58,8 +58,8 @@ ts_dimension_slice_scan_range_limit(int32 dimension_id, StrategyNumber start_str
 									int limit, const ScanTupLock *tuplock);
 extern DimensionVec *ts_dimension_slice_collision_scan_limit(int32 dimension_id, int64 range_start,
 															 int64 range_end, int limit);
-extern bool ts_dimension_slice_scan_for_existing(const DimensionSlice *slice,
-												 const ScanTupLock *tuplock);
+extern TSDLLEXPORT bool ts_dimension_slice_scan_for_existing(const DimensionSlice *slice,
+															 const ScanTupLock *tuplock);
 extern DimensionSlice *ts_dimension_slice_scan_by_id_and_lock(int32 dimension_slice_id,
 															  const ScanTupLock *tuplock,
 															  MemoryContext mctx,
@@ -70,18 +70,20 @@ extern DimensionVec *ts_dimension_slice_scan_by_dimension_before_point(int32 dim
 																	   ScanDirection scandir,
 																	   MemoryContext mctx);
 extern int ts_dimension_slice_delete_by_dimension_id(int32 dimension_id, bool delete_constraints);
-extern int ts_dimension_slice_delete_by_id(int32 dimension_slice_id, bool delete_constraints);
+extern TSDLLEXPORT int ts_dimension_slice_delete_by_id(int32 dimension_slice_id,
+													   bool delete_constraints);
 extern TSDLLEXPORT DimensionSlice *ts_dimension_slice_create(int dimension_id, int64 range_start,
 															 int64 range_end);
 extern TSDLLEXPORT DimensionSlice *ts_dimension_slice_copy(const DimensionSlice *original);
 extern TSDLLEXPORT bool ts_dimension_slices_collide(const DimensionSlice *slice1,
 													const DimensionSlice *slice2);
-extern bool ts_dimension_slices_equal(const DimensionSlice *slice1, const DimensionSlice *slice2);
+extern TSDLLEXPORT bool ts_dimension_slices_equal(const DimensionSlice *slice1,
+												  const DimensionSlice *slice2);
 extern bool ts_dimension_slice_cut(DimensionSlice *to_cut, const DimensionSlice *other,
 								   int64 coord);
 extern void ts_dimension_slice_free(DimensionSlice *slice);
 extern int ts_dimension_slice_insert_multi(DimensionSlice **slice, Size num_slices);
-extern void ts_dimension_slice_insert(DimensionSlice *slice);
+extern TSDLLEXPORT void ts_dimension_slice_insert(DimensionSlice *slice);
 extern int ts_dimension_slice_cmp(const DimensionSlice *left, const DimensionSlice *right);
 extern int ts_dimension_slice_cmp_coordinate(const DimensionSlice *slice, int64 coord);
 
