@@ -240,6 +240,7 @@ Hypertable *
 ts_hypertable_from_tupleinfo(const TupleInfo *ti)
 {
 	Hypertable *h = MemoryContextAllocZero(ti->mctx, sizeof(Hypertable));
+	char relkind;
 
 	ts_hypertable_formdata_fill(&h->fd, ti);
 	h->main_table_relid =
@@ -249,8 +250,14 @@ ts_hypertable_from_tupleinfo(const TupleInfo *ti)
 		ts_subspace_store_init(h->space, ti->mctx, ts_guc_max_cached_chunks_per_hypertable);
 	h->chunk_sizing_func = get_chunk_sizing_func_oid(&h->fd);
 
-	h->range_space =
-		ts_chunk_column_stats_range_space_scan(h->fd.id, h->main_table_relid, ti->mctx);
+	if (OidIsValid(h->main_table_relid))
+		ts_get_rel_info(h->main_table_relid, &h->amoid, &relkind);
+
+	if (ts_guc_enable_chunk_skipping)
+	{
+		h->range_space =
+			ts_chunk_column_stats_range_space_scan(h->fd.id, h->main_table_relid, ti->mctx);
+	}
 
 	return h;
 }
@@ -924,7 +931,7 @@ hypertable_insert(int32 hypertable_id, Name schema_name, Name table_name,
 static ScanTupleResult
 hypertable_tuple_found(TupleInfo *ti, void *data)
 {
-	Hypertable **entry = data;
+	Hypertable **entry = (Hypertable **) data;
 
 	*entry = ts_hypertable_from_tupleinfo(ti);
 	return SCAN_DONE;
@@ -935,7 +942,7 @@ ts_hypertable_get_by_name(const char *schema, const char *name)
 {
 	Hypertable *ht = NULL;
 
-	hypertable_scan(schema, name, hypertable_tuple_found, &ht, AccessShareLock);
+	hypertable_scan(schema, name, hypertable_tuple_found, (void *) &ht, AccessShareLock);
 
 	return ht;
 }
@@ -956,7 +963,7 @@ ts_hypertable_get_by_id(int32 hypertable_id)
 								   1,
 								   HYPERTABLE_ID_INDEX,
 								   hypertable_tuple_found,
-								   &ht,
+								   (void *) &ht,
 								   1,
 								   AccessShareLock,
 								   CurrentMemoryContext,
@@ -1922,13 +1929,10 @@ ts_hypertable_create_from_info(Oid table_relid, int32 hypertable_id, uint32 flag
 		hypertable_create_schema(NameStr(*associated_schema_name));
 
 	/*
-	 * Hypertables do not support transition tables in triggers, so if the
-	 * table already has such triggers we bail out
+	 * Hypertables do not support arbitrary triggers, so if the table already
+	 * has unsupported triggers we bail out
 	 */
-	if (ts_relation_has_transition_table_trigger(table_relid))
-		ereport(ERROR,
-				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("hypertables do not support transition tables in triggers")));
+	ts_check_unsupported_triggers(table_relid);
 
 	if (NULL == chunk_sizing_info)
 		chunk_sizing_info = ts_chunk_sizing_info_get_default_disabled(table_relid);
