@@ -10,6 +10,3 @@
 #include <optimizer/planner.h>
 
 #include "chunk.h"
-
-void ts_pushdown_partial_agg(PlannerInfo *root, Hypertable *ht, RelOptInfo *input_rel,
-							 RelOptInfo *output_rel, void *extra);
