@@ -12,6 +12,7 @@
 #include <miscadmin.h>
 #include <nodes/makefuncs.h>
 #include <nodes/nodeFuncs.h>
+#include <nodes/parsenodes.h>
 #include <nodes/plannodes.h>
 #include <optimizer/appendinfo.h>
 #include <optimizer/clauses.h>
@@ -55,6 +56,7 @@
 #include "partitioning.h"
 #include "planner/partialize.h"
 #include "planner/planner.h"
+#include "sort_transform.h"
 #include "utils.h"
 
 #include "compat/compat.h"
@@ -653,7 +655,7 @@ timescaledb_planner(Query *parse, const char *query_string, int cursor_opts,
 			preprocess_query((Node *) parse, &context);
 
 			if (ts_guc_enable_optimizations)
-				ts_cm_functions->preprocess_query_tsl(parse);
+				ts_cm_functions->preprocess_query_tsl(parse, &cursor_opts);
 		}
 
 		if (prev_planner_hook != NULL)
@@ -934,8 +936,6 @@ ts_classify_relation(const PlannerInfo *root, const RelOptInfo *rel, Hypertable
 	return TS_REL_OTHER;
 }
 
-extern void ts_sort_transform_optimization(PlannerInfo *root, RelOptInfo *rel);
-
 static inline bool
 should_chunk_append(Hypertable *ht, PlannerInfo *root, RelOptInfo *rel, Path *path, bool ordered,
 					int order_attno)
@@ -1182,14 +1182,47 @@ apply_optimizations(PlannerInfo *root, TsRelType reltype, RelOptInfo *rel, Range
 			break;
 		case TS_REL_CHUNK_STANDALONE:
 		case TS_REL_CHUNK_CHILD:
-			ts_sort_transform_optimization(root, rel);
+		{
 			/*
 			 * Since the sort optimization adds new paths to the rel it has
 			 * to happen before any optimizations that replace pathlist.
 			 */
-			if (ts_cm_functions->set_rel_pathlist_query != NULL)
-				ts_cm_functions->set_rel_pathlist_query(root, rel, rel->relid, rte, ht);
+			List *transformed_query_pathkeys = ts_sort_transform_get_pathkeys(root, rel, rte, ht);
+			if (transformed_query_pathkeys != NIL)
+			{
+				List *orig_query_pathkeys = root->query_pathkeys;
+				root->query_pathkeys = transformed_query_pathkeys;
+
+				/* Create index paths with transformed pathkeys */
+				create_index_paths(root, rel);
+
+				/*
+				 * Call the TSL hooks with the transformed pathkeys as well, so
+				 * that the decompression paths also use this optimization.
+				 */
+				if (ts_cm_functions->set_rel_pathlist_query != NULL)
+					ts_cm_functions->set_rel_pathlist_query(root, rel, rel->relid, rte, ht);
+
+				root->query_pathkeys = orig_query_pathkeys;
+
+				/*
+				 * change returned paths to use original pathkeys. have to go through
+				 * all paths since create_index_paths might have modified existing
+				 * pathkey. Always safe to do transform since ordering of
+				 * transformed_query_pathkey implements ordering of
+				 * orig_query_pathkeys.
+				 */
+				ts_sort_transform_replace_pathkeys(rel->pathlist,
+												   transformed_query_pathkeys,
+												   orig_query_pathkeys);
+			}
+			else
+			{
+				if (ts_cm_functions->set_rel_pathlist_query != NULL)
+					ts_cm_functions->set_rel_pathlist_query(root, rel, rel->relid, rte, ht);
+			}
 			break;
+		}
 		default:
 			break;
 	}
@@ -1272,8 +1305,15 @@ timescaledb_set_rel_pathlist(PlannerInfo *root, RelOptInfo *rel, Index rti, Rang
 	TsRelType reltype;
 	Hypertable *ht;
 
-	/* Quick exit if this is a relation we're not interested in */
-	if (!valid_hook_call() || !OidIsValid(rte->relid) || IS_DUMMY_REL(rel))
+	/*
+	 * Quick exit if this is a relation we're not interested in.
+	 *
+	 * If the rtekind is a named tuple store, it is a named tuple store *for*
+	 * the relation rte->relid (e.g., a transition table for a trigger), but
+	 * not the relation itself.
+	 */
+	if (!valid_hook_call() || rte->rtekind == RTE_NAMEDTUPLESTORE || !OidIsValid(rte->relid) ||
+		IS_DUMMY_REL(rel))
 	{
 		if (prev_set_rel_pathlist_hook != NULL)
 			(*prev_set_rel_pathlist_hook)(root, rel, rti, rte);
@@ -1417,11 +1457,13 @@ timescaledb_get_relation_info_hook(PlannerInfo *root, Oid relation_objectid, boo
 											 !TS_HYPERTABLE_IS_INTERNAL_COMPRESSION_TABLE(ht);
 			const bool is_child_chunk_in_update =
 				(type == TS_REL_CHUNK_CHILD) && IS_UPDL_CMD(query);
+
 			if (use_transparent_decompression && (is_standalone_chunk || is_child_chunk_in_update))
 			{
 				const Chunk *chunk = ts_planner_chunk_fetch(root, rel);
 
-				if (!ts_chunk_is_partial(chunk) && ts_chunk_is_compressed(chunk))
+				if (!ts_chunk_is_partial(chunk) && ts_chunk_is_compressed(chunk) &&
+					!ts_is_hypercore_am(chunk->amoid))
 				{
 					rel->indexlist = NIL;
 				}
