@@ -12,6 +12,8 @@
 #include <access/xact.h>
 #include <catalog/indexing.h>
 #include <catalog/namespace.h>
+#include <catalog/objectaccess.h>
+#include <catalog/pg_am.h>
 #include <catalog/pg_cast.h>
 #include <catalog/pg_inherits.h>
 #include <catalog/pg_operator.h>
@@ -24,6 +26,7 @@
 #include <parser/parse_coerce.h>
 #include <parser/parse_func.h>
 #include <parser/scansup.h>
+#include <storage/lockdefs.h>
 #include <utils/acl.h>
 #include <utils/builtins.h>
 #include <utils/catcache.h>
@@ -37,6 +40,7 @@
 
 #include "compat/compat.h"
 #include "chunk.h"
+#include "cross_module_fn.h"
 #include "debug_point.h"
 #include "guc.h"
 #include "hypertable_cache.h"
@@ -290,9 +294,9 @@ ts_time_value_to_internal_or_infinite(Datum time_val, Oid type_oid)
 
 			return ts_time_value_to_internal(time_val, type_oid);
 		}
+		default:
+			return ts_time_value_to_internal(time_val, type_oid);
 	}
-
-	return ts_time_value_to_internal(time_val, type_oid);
 }
 
 TS_FUNCTION_INFO_V1(ts_time_to_internal);
@@ -1532,6 +1536,14 @@ ts_copy_relation_acl(const Oid source_relid, const Oid target_relid, const Oid o
 		new_repl[AttrNumberGetAttrOffset(Anum_pg_class_relacl)] = true;
 		new_val[AttrNumberGetAttrOffset(Anum_pg_class_relacl)] = PointerGetDatum(acl);
 
+		/*
+		 * ts_copy_relation_acl() is typically used to copy ACLs from the hypertable
+		 * to a newly created chunk. The creation is done via DefineRelation(),
+		 * which takes an AccessExclusiveLock and should be enough to handle any
+		 * inplace update issues.
+		 */
+		AssertSufficientPgClassUpdateLockHeld(target_relid);
+
 		/* Find the tuple for the target in `pg_class` */
 		target_tuple = SearchSysCache1(RELOID, ObjectIdGetDatum(target_relid));
 		Assert(HeapTupleIsValid(target_tuple));
@@ -1797,33 +1809,125 @@ ts_get_rel_info_by_name(const char *relnamespace, const char *relname, Oid *reli
 	ReleaseSysCache(tuple);
 }
 
+Oid
+ts_get_rel_am(Oid relid)
+{
+	HeapTuple tuple;
+	Form_pg_class cform;
+	Oid amoid;
+
+	tuple = SearchSysCache1(RELOID, ObjectIdGetDatum(relid));
+
+	if (!HeapTupleIsValid(tuple))
+		elog(ERROR, "cache lookup failed for relation %u", relid);
+
+	cform = (Form_pg_class) GETSTRUCT(tuple);
+	amoid = cform->relam;
+	ReleaseSysCache(tuple);
+
+	return amoid;
+}
+
 static Oid hypercore_amoid = InvalidOid;
 
 bool
 ts_is_hypercore_am(Oid amoid)
 {
-	/* Can't use InvalidOid as an indication of non-cached value since
-	   get_am_oid() will return InvalidOid when the access method does not
-	   exist and we will do the lookup every time this query is called. This
-	   boolean can be removed once we know that there should exist an access
-	   method with the given name. */
-	static bool iscached = false;
-
-	if (!iscached && !OidIsValid(hypercore_amoid))
-	{
-		hypercore_amoid = get_am_oid("hypercore", true);
-		iscached = true;
-	}
-
 	if (!OidIsValid(hypercore_amoid))
+		hypercore_amoid = get_table_am_oid(TS_HYPERCORE_TAM_NAME, true);
+
+	if (!OidIsValid(amoid) || !OidIsValid(hypercore_amoid))
 		return false;
 
-	/* Shouldn't get here for now */
-	Assert(false);
-
 	return amoid == hypercore_amoid;
 }
 
+/*
+ * Set reloption for relation.
+ *
+ * Most of the code is from ATExecSetRelOptions() in tablecmds.c since that
+ * function is static and we also need to do a slightly different job.
+ */
+static void
+relation_set_reloption_impl(Relation rel, List *options, LOCKMODE lockmode)
+{
+	Datum repl_val[Natts_pg_class] = { 0 };
+	bool repl_null[Natts_pg_class] = { false };
+	bool repl_repl[Natts_pg_class] = { false };
+	bool isnull;
+
+	Assert(rel->rd_rel->relkind == RELKIND_RELATION || rel->rd_rel->relkind == RELKIND_TOASTVALUE);
+
+	if (options == NIL)
+		return; /* nothing to do */
+
+	TS_DEBUG_LOG("setting reloptions for %s", RelationGetRelationName(rel));
+
+	Relation pgclass = table_open(RelationRelationId, RowExclusiveLock);
+	Oid relid = RelationGetRelid(rel);
+	HeapTuple tuple = SearchSysCacheLockedCopy1(RELOID, ObjectIdGetDatum(relid));
+	if (!HeapTupleIsValid(tuple))
+		elog(ERROR, "cache lookup failed for relation %u", relid);
+#ifdef SYSCACHE_TUPLE_LOCK_NEEDED
+	ItemPointerData otid = tuple->t_self;
+#endif
+
+	/* Get the old reloptions */
+	Datum datum = SysCacheGetAttr(RELOID, tuple, Anum_pg_class_reloptions, &isnull);
+
+	/* Generate new proposed reloptions (text array) */
+	Datum newOptions =
+		transformRelOptions(isnull ? (Datum) 0 : datum, options, NULL, NULL, false, false);
+	(void) heap_reloptions(rel->rd_rel->relkind, newOptions, true);
+
+	if (newOptions)
+		repl_val[AttrNumberGetAttrOffset(Anum_pg_class_reloptions)] = newOptions;
+	else
+		repl_null[AttrNumberGetAttrOffset(Anum_pg_class_reloptions)] = true;
+
+	repl_repl[AttrNumberGetAttrOffset(Anum_pg_class_reloptions)] = true;
+
+	HeapTuple newtuple =
+		heap_modify_tuple(tuple, RelationGetDescr(pgclass), repl_val, repl_null, repl_repl);
+
+	CatalogTupleUpdate(pgclass, &newtuple->t_self, newtuple);
+
+	/* Not sure if we need this one, but keeping it as a precaution */
+	InvokeObjectPostAlterHook(RelationRelationId, RelationGetRelid(rel), 0);
+
+	UnlockSysCacheTuple(pgclass, &otid);
+	heap_freetuple(newtuple);
+	heap_freetuple(tuple);
+	table_close(pgclass, RowExclusiveLock);
+}
+
+/*
+ * Set value of reloptions for given relation.
+ *
+ * This will also set the reloption for the relations' associated relations,
+ * in this case the TOAST table. It is based on ATExecSetRelOptions but we
+ * split out the code to set the reloptions rather than duplicating it.
+ *
+ * The lockmode is needed for taking a correct lock on the toast table for the
+ * already locked relation. It is only used for
+ *
+ * rel: Relation to add reloptions to.
+ * defList: List of DefElem for the new definitions.
+ * lockmode: the mode that the actual tables are locked in.
+ */
+void
+ts_relation_set_reloption(Relation rel, List *options, LOCKMODE lockmode)
+{
+	Assert(RelationIsValid(rel));
+	relation_set_reloption_impl(rel, options, lockmode);
+	if (OidIsValid(rel->rd_rel->reltoastrelid))
+	{
+		Relation toastrel = table_open(rel->rd_rel->reltoastrelid, lockmode);
+		relation_set_reloption_impl(toastrel, options, lockmode);
+		table_close(toastrel, NoLock);
+	}
+}
+
 /* this function fills in a jsonb with the non-null fields of
  the error data and also includes the proc name and schema in the jsonb
  we include these here to avoid adding these fields to the table */
